@@ -32,7 +32,7 @@ from shapely.geometry import box, LineString, Point
 from typing import Union, List
 from gemgis import vector
 import pyproj
-import pygeos
+import pyvista as pv
 
 __all__ = [series, crs]
 
@@ -319,7 +319,7 @@ def set_extent(minx: Union[int, float] = 0,
             extent = [minx, maxx, miny, maxy, minz, maxz]
 
     # Create extent from gdf of geom_type polygon
-    elif all(pygeos.get_type_id(pygeos.from_shapely(gdf.geometry)) == 2):
+    elif all(shapely.get_type_id(gdf.geometry) == 2):
 
         # Checking if the gdf is of type GeoDataFrame
         bounds = gdf.bounds.round().values.tolist()[0]
@@ -1747,3 +1747,396 @@ def interpolate_strike_lines(gdf: gpd.geodataframe.GeoDataFrame,
     gdf_out['id'] = np.arange(1, len(gdf_out['id'].values.tolist()) + 1).tolist()
 
     return gdf_out
+
+
+def convert_to_petrel_points_with_attributes(mesh: pv.core.pointset.PolyData,
+                                             path: str,
+                                             crs: Union[str, pyproj.crs.crs.CRS, type(None)] = None,
+                                             target_crs: Union[str, pyproj.crs.crs.CRS, type(None)] = None):
+    """Function to convert vertices of a PyVista Mesh to Petrel Points with Attributes
+
+    Parameters:
+    ___________
+
+        mesh: pv.core.pointset.PolyData
+            PyVista Mesh to be converted to points
+
+        path: str
+            Path to store the converted points
+
+        crs: str, pyproj.crs.crs.CRS, type(None)
+            Coordinate reference system for the GeoDataFrame
+
+        target_crs: str, pyproj.crs.crs.CRS, type(None)
+            Target coordinate reference system if coordinates of points should be reprojected
+
+
+    """
+
+    # Checking that the mesh is a PyVista PolyData object
+    if not isinstance(mesh, pv.core.pointset.PolyData):
+        raise TypeError('Mesh must be provided as PyVista PolyData object')
+
+    # Checking that the CRS is provided as proper type
+    if not isinstance(crs, (str, pyproj.crs.crs.CRS, type(None))):
+        raise TypeError('CRS must be provided as string or pyproj CRS object')
+
+    # Checking that the target CRS is provided as proper type
+    if not isinstance(target_crs, (str, pyproj.crs.crs.CRS, type(None))):
+        raise TypeError('CRS must be provided as string or pyproj CRS object')
+
+    # Selecting vertices
+    vertices = np.array(mesh.points)
+
+    # Creating GeoDataFrame from vertices
+    gdf = gpd.GeoDataFrame(geometry=gpd.points_from_xy(vertices[:, 0], vertices[:, 1]), data=vertices,
+                           columns=['X', 'Y', 'Z'], crs=crs)
+
+    # Reprojecting data and extracting X and Y coordinates
+    if target_crs and target_crs != crs:
+        gdf = gdf.to_crs(crs=target_crs)
+        gdf = vector.extract_xy(gdf=gdf)
+
+    # Dropping Geometry Column
+    df = gdf.drop('geometry', axis=1)
+
+    df.to_csv(fname=path,
+              index=False,
+              sep='\t')
+
+    print('CSV-File successfully saved')
+
+
+def ray_trace_one_surface(surface: pv.core.pointset.PolyData,
+                          origin: Union[np.ndarray, list],
+                          end_point: Union[np.ndarray, list],
+                          first_point: bool = False) -> tuple:
+    """Function to return the depth of one surface in one well using PyVista ray tracing
+
+    Parameters:
+    ___________
+
+        surface: pv.core.pointset.PolyData
+            Calculated GemPy surface
+
+        origin:
+            Coordinates of the top of the well
+
+        end_point:
+            Coordinates of the bottom of the well
+
+        first_point: bool
+            Returns intersection of first point only
+
+    """
+
+    # Extracting the intersection between a PolyData set and a mesh
+    intersection_points, intersection_cells = surface.ray_trace(origin=origin,
+                                                                end_point=end_point,
+                                                                first_point=first_point)
+
+    return intersection_points, intersection_cells
+
+
+def ray_trace_multiple_surfaces(surfaces: list,
+                                borehole_top: Union[np.ndarray, list],
+                                borehole_bottom: Union[np.ndarray, list],
+                                first_point: bool = False) -> list:
+    """Function to return the depth of multiple surfaces in one well using PyVista ray tracing
+
+    Parameters:
+    ___________
+
+        surfaces: list
+            List of calculated GemPy surfaces
+
+        borehole_top:
+            Coordinates of the top of the well
+
+        borehole_bottom:
+            Coordinates of the bottom of the well
+
+        first_point: bool
+            Returns intersection of first point only
+
+
+    """
+
+    # Extracting multiple intersections from meshes
+    intersections = [ray_trace_one_surface(surface=surface,
+                                           origin=borehole_top,
+                                           end_point=borehole_bottom,
+                                           first_point=first_point) for surface in surfaces]
+
+    return intersections
+
+
+def create_virtual_profile(names_surfaces: list,
+                           surfaces: list,
+                           borehole: pv.core.pointset.PolyData,
+                           first_point: bool = False):
+    """ Function to filter and sort the resulting well tops
+
+    Parameters:
+    ___________
+
+        names_surfaces: list
+            List of the names of the calculated GemPy surfaces
+
+        surfaces: list
+            List of calculated GemPy surfaces
+
+        borehole_top:
+            Coordinates of the top of the well
+
+        borehole_bottom:
+            Coordinates of the bottom of the well
+
+        first_point: bool
+            Returns intersection of first point only
+
+    """
+
+    # Creating well segments
+    well_segments = [pv.Line(borehole.points[i], borehole.points[i + 1]) for i in range(len(borehole.points) - 1)]
+
+    # Extracting well tops
+    well_tops = [ray_trace_multiple_surfaces(surfaces=surfaces,
+                                             borehole_top=segment.points[0],
+                                             borehole_bottom=segment.points[1],
+                                             first_point=first_point) for segment in well_segments]
+
+    # Flatten list
+    well_tops = [item for sublist in well_tops for item in sublist]
+
+    # Filtering empty lists
+    well_tops_filtered = []
+    list_surfaces = names_surfaces * len(well_segments)
+    list_surfaces_filtered = []
+
+    for i in range(len(well_tops)):
+        if len(well_tops[i][0] != 0):
+            well_tops_filtered.append(well_tops[i])
+            list_surfaces_filtered.append(list_surfaces[i])
+
+    # Extracting Z values
+    z_values = [values[0][0][2] for values in well_tops_filtered]
+
+    # Creating dict from names and well tops
+    #well_dict = dict(zip(list_surfaces_filtered, well_tops_filtered))
+
+    # Removing empty entries
+    #for key in well_dict.copy():
+    #    if not well_dict[key][0].any():
+    #        well_dict.pop(key)
+
+    # Splitting layers if they have been encountered more than once
+    #for key in well_dict.copy():
+    #    if len(well_dict[key][0]) > 1:
+    #        for i in range(1, len(well_dict[key][0])):
+    #            well_dict[key + '_%s' % str(i + 1)] = (well_dict[key][0][i].reshape(1, 3), well_dict[key][1][i])
+
+    # Extracting only Z value from dict
+    #for key in well_dict.keys():
+    #    well_dict[key] = round(well_dict[key][0][0][2])
+
+    # Sorting well dict
+    #well_dict = dict(sorted(well_dict.items(), key=lambda item: item[1], reverse=True))
+
+    # Creating DataFrame
+    df = pd.DataFrame(list(zip(list_surfaces_filtered, z_values)), columns=['Surface', 'Z'])
+
+    return df
+
+
+def extract_zmap_data(surface: pv.core.pointset.PolyData,
+                      cell_width: int,
+                      nodata: Union[float, int] = -9999):
+    """Function to extract a meshgrid of values from a PyVista mesh
+
+    Parameters:
+    ___________
+
+        surface: pv.core.pointset.PolyData
+            PyVista mesh
+
+        cell_width: int
+            Width of grid cell
+
+        nodata: Union[float, int]
+            No data value
+
+    """
+
+    # Extracting extent
+    extent = surface.bounds
+
+    # Calculating x dimension
+    x_dim = extent[1] - extent[0]
+
+    # Calculating y dimension
+    y_dim = extent[3] - extent[2]
+
+    # Calculate number of cells in x direction
+    x_no_cells = round(x_dim / cell_width)
+
+    # Calculate number of cells in y direction
+    y_no_cells = round(y_dim / cell_width)
+
+    # Calculate coordinates in x direction
+    x = np.arange(extent[0] + 0.5 * cell_width, extent[1], cell_width)
+
+    # Calculate coordinates in y direction
+    y = np.arange(extent[2] + 0.5 * cell_width, extent[3], cell_width)
+
+    # Calculating the intersections
+    intersections = [ray_trace_one_surface(surface=surface,
+                                           origin=[x_value, y_value, extent[4]],
+                                           end_point=[x_value, y_value, extent[5]],
+                                           first_point=True) for x_value in x for y_value in y]
+
+    # Extracting the height values
+    z_values = np.array([z[0][2] if len(z[0]) == 3 else nodata for z in intersections]).reshape(x_no_cells,
+                                                                                                y_no_cells).T
+
+    return z_values
+
+
+def create_zmap_grid(surface: pv.core.pointset.PolyData,
+                     cell_width: int,
+                     comments: str = '',
+                     name: str = 'ZMAP_Grid',
+                     z_type: str = 'GRID',
+                     nodes_per_line: int = 5,
+                     field_width: int = 15,
+                     nodata: Union[int, float] = -9999.00000,
+                     nodata2: Union[int, float, str] = '',
+                     decimal_places: int = 5,
+                     start_column: int = 1):
+    """Function to write data to ZMAP Grid, This code is heavily inspired by https://github.com/abduhbm/zmapio
+
+    Parameters:
+    ___________
+
+        surface: pv.core.pointset.PolyData
+            PyVista mesh
+
+        cell_width: int
+            Width of grid cell
+
+        comments: str
+            Comments written to the ZMAP File
+
+        name: str
+            Name of the ZMAP File
+
+        z_type: str
+            ZMAP Grid Type
+
+        nodes_per_lines: int
+            Number of values per line
+
+        field_width: int
+            Width of each field
+
+        nodata: Union[int, float]
+            No data value
+
+        nodata2:  Union[int, float, str]
+            No data value
+
+        decimal_places: int
+            Number of Decimal Places
+
+        start_column: int
+            Number of the start column
+    """
+
+    # Extracting z_values
+    z_values = extract_zmap_data(surface=surface,
+                                 cell_width=cell_width,
+                                 nodata=nodata)
+
+    # Defining extent
+    extent = surface.bounds
+
+    # Defining the number of rows and columns
+    no_cols = z_values.shape[1]
+    no_rows = z_values.shape[0]
+
+    # Defining auxiliary function
+    def chunks(x, n):
+        for i in range(0, len(x), n):
+            yield x[i: i + n]
+
+    # Create list of lines with first comments
+    lines = ['!', '! This ZMAP Grid was created using the GemGIS Package',
+             '! See https://github.com/cgre-aachen/gemgis for more information', '!']
+
+    # Appending comments to lines
+    for comment in comments:
+        lines.append('! ' + comment)
+    lines.append('!')
+
+    # Appending header information to lines
+    lines.append("@{}, {}, {}".format(name, z_type, nodes_per_line))
+
+    # Appending header information to lines
+    lines.append(
+        "{}, {}, {}, {}, {}".format(
+            field_width,
+            nodata,
+            nodata2,
+            decimal_places,
+            start_column,
+        )
+    )
+
+    # Appending header information to lines
+    lines.append(
+        "{}, {}, {}, {}, {}, {}".format(
+            no_rows,
+            no_cols,
+            extent[0],
+            extent[1],
+            extent[2],
+            extent[3]
+        )
+    )
+
+    # Appending header information to lines
+    lines.append("0.0, 0.0, 0.0")
+    lines.append("@")
+
+    # Appending z values to lines
+    for i in z_values:
+        for j in chunks(i, nodes_per_line):
+            j_fmt = "0.{}f".format(decimal_places)
+            j_fmt = "{0:" + j_fmt + "}"
+            j = [j_fmt.format(float(x)) if not x is np.nan else j_fmt.format(float(nodata)) for x in j]
+            line = "{:>" + "{}".format(field_width) + "}"
+            lines.append("".join([line] * len(j)).format(*tuple(j)))
+
+    return lines
+
+
+def save_zmap_grid(zmap_grid: list,
+                   path: str = 'ZMAP_Grid.dat'):
+    """Function to save ZMAP Grid information to file
+
+    Parameters:
+    ___________
+
+        zmap_grid: list
+            List of strings containing the ZMAP Data
+
+        path: str
+            Path and filename to store the ZMAP Grid
+
+    """
+
+    # Writing the ZMAP Grid to file
+    with open(path, 'w') as f:
+        f.write("\n".join(zmap_grid))
+
+    print('ZMAP Grid successfully saved to file')
